#include	<pic.h>
#include	"delay\delay.c"
#include	"usart\sci.c"
#include	"DEFINE.h"

// Define os fusiveis do PIC
__CONFIG	(PROTECT & CPD & LVPDIS & BOREN & MCLREN & PWRTEN & WDTDIS & INTIO);


unsigned char Tabela7seg(char a);
void interrupt InterrupcaoALTA (void);
void	Limpa_Erros_USART	(void);
void	Varre7seg (char *a, char b);
void	LimpaArray (char *b, char c);
char getche(void);
void load_portas(void);





/************************************************************************
/ Funcao de tratamento de Interrupcoes
/************************************************************************/
void interrupt Interrupcao (void)
{
	//entra se houve e estah autorizada interrupcao de TMR0
	if	((T0IF == 1) && (T0IE == 1))
	{
		T0IF = 0;		//flag de interrupcao de TMR0
		
		Varre7seg (Array7seg, ContadorPosicao7seg);	//mostra o digito correspondente

		ContadorPosicao7seg++;			//incrementa digito 7seg
		if	(ContadorPosicao7seg == 4)
			ContadorPosicao7seg = 0;
	}
	

	//entra se houve e estah autorizada interrupcao de USART
	if	((RCIF == 1) && (RCIE == 1))
	{
		DataSerieBuf[ContSerieBuff] = getche();	//guarda o byte recebido no array de chegada, manda ECO
		ContSerieBuff++;		//incrementa o contador p a prox posicao do array de chegada
	
		FLAGS.USARTRX = 1;		//assinala a recepcao de dados USART
	
		if	(ContSerieBuff >= StringBuff)		//se o contador chegou ao fim, poe-o no inicio
			ContSerieBuff = 0;
	}
}






/****************************************************************************/
/*	Programa principal
/****************************************************************************/
main(void)
{
	char	i;	//temporario

	PORTA = PORTB = 0;		//configuracao das portas
	TRISA = 0b00100000; 
	TRISB = 0b00000110;
	PORTA = PORTB = 0;
	
	CMCON = 0x07;		//PORTA toda digital
		
	//Configura esquema de Interrupcoes
	GIE = 1;		//liga interrupcoes
	PEIE = 1;		//liga interrupcoes de perifericos

//	RBPU = 1;	//liga Pull-ups na porta B

	//Configura interrupcao da tecla em RB0
//	INTEDG = 0;		//interrupcao no flanco descendente
//	INTE = 0;		//interrupcao INT0
//	INTF = 0;		//flag interrupcao INT0

	// Configura USART
	sci_Init (9600, SCI_EIGHT);
	TXEN = 1;		//ligar transmissao
	RCIE = 1;		//interrupcao de Recepcao USART

	//Configura TMR0
	T0CS = 0;		//TMR0 incrementa com relogio interno
	PSA = 0;		//prescaler ligado ao TMR0
	PS2 = 0;		//1:16	~245Hz
	PS1 = 1;		//
	PS0 = 1;		//
	TMR0 = 0;		//inicializa o contador
	T0IE = 1;		//interrupcao de TMR0
	T0IF = 0;		//flag de interrupcao de TMR0
	
/*	Faz a limpeza dos erros da USART, caso os haja. Tenta evitar crashs do sistema
	Alem de limpar os erros, eh aqui q liga a recepcao continua da USART			*/
	Limpa_Erros_USART();



	while(1)
	{

		//entra se recebeu informacao pela porta serie
		if	(FLAGS.USARTRX)
		{
			FLAGS.USARTRX = 0;	//limpa a flag
			
			//Faz a limpeza dos erros da USART, caso os haja. Tenta evitar crashs do sistema.
			Limpa_Erros_USART();
			
			//entra se recebeu o ultimo byte da trama
			if (DataSerieBuf[7] == '!')
			{
				ContSerieBuff = 0;	//inicializa o contador do buffer da porta serie

				for	(i=0;i<6;i++)
					ContadorCRC += DataSerieBuf[i];		//soma os 6 bytes de interesse
				
				//entra se o CRC estiver correcto
				if	(ContadorCRC == DataSerieBuf[6])
				{
					Array7seg[3] = (DataSerieBuf[2] & 0x0F);	//ASCII para BCD
					Array7seg[2] = (DataSerieBuf[3] & 0x0F);
					Array7seg[1] = (DataSerieBuf[4] & 0x0F);
					Array7seg[0] = (DataSerieBuf[5] & 0x0F);

					LimpaArray (DataSerieBuf,StringBuff);		//limpa buffer da porta serie
					ContadorCRC = 0;	//limpa contador CRC
				}
			}
		}//if	(FLAGS.USARTRX)
		
	}//while(1)
	
}

/****************************************************************************/
/*	Fim de Programa principal
/****************************************************************************/

//*#99991!
//*#4567#!
//*#4568$!










/********************************************************************
*     Function Name:    Tabela7seg									*
*     Parameters:       numero a mostrar							*
*     Description:      Converte de BCD para 7segmentos				*
********************************************************************/
unsigned char Tabela7seg(char a)
{
	switch	(a)
	{
		case 0:		return 0b00111111;	//converte de BINARIO para 7-SEGMENTOS
		case 1:		return 0b00000110;	//converte de BINARIO para 7-SEGMENTOS
		case 2:		return 0b01011011;	//converte de BINARIO para 7-SEGMENTOS
		case 3:		return 0b01001111;	//converte de BINARIO para 7-SEGMENTOS
		case 4:		return 0b01100110;	//converte de BINARIO para 7-SEGMENTOS
		case 5:		return 0b01101101;	//converte de BINARIO para 7-SEGMENTOS
		case 6:		return 0b01111101;	//converte de BINARIO para 7-SEGMENTOS
		case 7:		return 0b00000111;	//converte de BINARIO para 7-SEGMENTOS
		case 8:		return 0b01111111;	//converte de BINARIO para 7-SEGMENTOS
		case 9:		return 0b01101111;	//converte de BINARIO para 7-SEGMENTOS
		case 0x0A:	return 0b01110111;	//converte de BINARIO para 7-SEGMENTOS
		case 0x0B:	return 0b01111100;	//converte de BINARIO para 7-SEGMENTOS
		case 0x0C:	return 0b01011000;	//converte de BINARIO para 7-SEGMENTOS
		case 0x0D:	return 0b01011110;	//converte de BINARIO para 7-SEGMENTOS
		case 0x0E:	return 0b01111001;	//converte de BINARIO para 7-SEGMENTOS
		case 0x0F:	return 0b01110001;	//converte de BINARIO para 7-SEGMENTOS
		default:	return 0b01000000;	//Se nao for nenhum destes, acende o segmento G
	}
}






/********************************************************************
*     Function Name:    Varre7seg									*
*     Parameters:       Array a mostrar, digito a mostrar			*
*     Description:      Mostra 4 digitos de 7segmentos				*
********************************************************************/
void	Varre7seg (char *a, char b)
{
	tempa = tempb = 0;	//limpa temporarios
	
	switch	(b)
	{
		case 0:	{
					if	( (Tabela7seg(a[b]) & 0x02) == 0x02 )
						bitset(tempa,6);	//segmento b
					if	( (Tabela7seg(a[b]) & 0x04) == 0x04 )
						bitset(tempa,7);	//segmento c
					bitset(tempa,0);	//liga digito 0
					tempb = Tabela7seg(a[b]);
					load_portas();
					break;
				}
		case 1:	{
					if	( (Tabela7seg(a[b]) & 0x02) == 0x02 )
						bitset(tempa,6);	//segmento b
					if	( (Tabela7seg(a[b]) & 0x04) == 0x04 )
						bitset(tempa,7);	//segmento c
					bitset(tempa,1);	//liga digito 1
					tempb = Tabela7seg(a[b]);
					load_portas();
					break;
				}
		case 2:	{
					if	( (Tabela7seg(a[b]) & 0x02) == 0x02 )
						bitset(tempa,6);	//segmento b
					if	( (Tabela7seg(a[b]) & 0x04) == 0x04 )
						bitset(tempa,7);	//segmento c
					bitset(tempa,2);	//liga digito 2
					tempb = Tabela7seg(a[b]);
					load_portas();
					break;
				}
		case 3:	{	
					if	( (Tabela7seg(a[b]) & 0x02) == 0x02 )
						bitset(tempa,6);	//segmento b
					if	( (Tabela7seg(a[b]) & 0x04) == 0x04 )
						bitset(tempa,7);	//segmento c
					bitset(tempa,3);	//liga digito 3
					tempb = Tabela7seg(a[b]);
					load_portas();
					break;
				}
		default:{
					break;
				}
	}


}

void load_portas(void)
{
	bitclr(tempb,1);	//desliga bits da porta serie
	bitclr(tempb,2);	//desliga bits da porta serie
	PORTB = tempb;
	PORTA = tempa;
}



/********************************************************************
*     Function Name:    LimpaArray									*
*     Parameters:       Array a limpar, numero de posicoes          *
*     Description:      Faz a configuracao do sistema               *
********************************************************************/
void	LimpaArray (char *b, char c)
{
	unsigned char a;			//cria contador temporario

	for( a=0 ; a<c ; a++ )		//apaga o array de chegada
		*(b+a) = '\0';
}


/********************************************************************
*     Function Name:    Limpa_Erros_USART							*
*     Parameters:                                                   *
*     Description:      Limpa os erros da USART                     *
********************************************************************/
void	Limpa_Erros_USART	(void)
{
//	while	(FERR)
//		EEDATA = RCREG;		//limpa o Framing Error Bit	(EEDATA como temporario)
	while	(OERR)
		CREN = 0;			//limpa o Overrun Error Bit
	CREN = 1;				//activa a recepcao continua de dados pela USART
}









/*	//Configura TMR1
	T1CON = 0b10000111;		//16-bit, EXT CLK Source, 1:1, OSC OFF
	TMR1 = 0;				//limpa o contador
	TMR1IF = 0;				//Clear overflow flag
	TMR1IP = 0;				//Baixa prioridade de Interrupcao
	TMR1IE = 0;				//Disable TIMER1 interrupts
*/
